package cologne.eck.peafactory.peas.gui;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Parent class of all pea frames except image pea. 
 */


import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.util.Arrays;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.peas.PswDialogBase;
import cologne.eck.peafactory.tools.Comparator;
import cologne.eck.peafactory.tools.ReadResources;
import cologne.eck.peafactory.tools.WriteResources;
import cologne.eck.peafactory.tools.Zeroizer;

@SuppressWarnings("serial")
public abstract class LockFrame extends JFrame implements WindowListener {
	

	/**
	 * Actions to perform when the main frame is closed (the program exits): 
	 * ask to save modifications, overwrite sensitive data...
	 */
	protected abstract void windowClosingCommands();	
	protected abstract byte[] getPlainBytes();

	/**
	 * Encrypt and store the current content of a file with a key. The key
	 * is then stored encrypted as session key. 
	 * 
	 * @param keyMaterial	the key to encrypt
	 * @param plainBytes	the content to encrypt 
	 * @param fileName		the file to store the encrypted content
	 * 
	 * @return	true if content was saved successfully
	 */
	public final static boolean saveContent(byte[] keyMaterial, byte[] plainBytes, String fileName) {
		
		//System.out.println("LockFrame saveContent, fileName: " + fileName);
		//Help.printlastBytes("LockFrame: plainBytes", plainBytes, 64);
		//Help.printBytes("LockFrame: keyMaterial", keyMaterial);
		
  /*      Window win = null;
        if (PswDialogView.getView().isVisible() == true) {
        	win = PswDialogView.getView();
        } else {
        	win = PswDialogBase.getDialog().getLockFrame();
        }
        // check if  the cursor was already set to wait cursor or not
        boolean cursorSet = false;
        if (win.getCursor().equals(CursorManager.getWaitCursor())){
        	cursorSet = true;
        }
		try {
			// set the wait cursor: 
			if (cursorSet == true) {
				win.setCursor(CursorManager.getWaitCursor());
			} */

			if (new File(fileName).isFile() == false) {
				System.err.println("LockFrame wrong file");
				return false;
			}
			
			byte[] cipherBytes = CipherStuff.getInstance().encrypt(plainBytes, 
					keyMaterial, // keyMaterial is overwritten
					true ); // key is stored encrypted
			
			if (cipherBytes == null) {
				return false;
			}
				
			boolean success = WriteResources.write(cipherBytes,fileName, null);
			return success;
/*		} finally {
			// reset the cursor:
			if (cursorSet == true) {
				win.setCursor(CursorManager.getDefaultCursor());
			}
		} */
	}
	/**
	 * Get a new password from a password dialog, derive a new key, 
	 * encrypt and save the content of files with the new key. 
	 * The new key will then be stored encrypted as session key and 
	 * will replace a previously stored key. 
	 * A wait cursor is set inside this function during the process.
	 * 
	 * @param fileNames		the names of other files to change the password, 
	 * 						the content of the files must be encrypted
	 * 						with the current session key
	 */
	public final void changePassword(String[] fileNames){

		// get the currently used key to decrypt several encrypted files
		// unfortunately the key must be present for a while... but this is
		// better than storing files in plain text for a while
		byte[] oldKeyMaterial = null;
		if (fileNames != null) {
			oldKeyMaterial = CipherStuff.getInstance().getSessionKeyCrypt().getKey();
			Comparator.checkNullVector(oldKeyMaterial);
		}
		
		// get the new password:
		NewPasswordDialog newPswDialog = NewPasswordDialog.getInstance(this, 
				PswDialogBase.getBundle().getString("change_password"), null,
				PswDialogView.getView().getLocation());
		char[] newPsw = newPswDialog.getDialogInput();	
		byte[] newKeyMaterial = null;
		try {
			// set the wait cursor: 
			PswDialogBase.getDialog().getLockFrame().setCursor(CursorManager.getWaitCursor());

			// derive the new key from the new password, using the same salt
			newKeyMaterial = PswDialogBase.deriveKeyFromPsw(newPsw);
			Comparator.checkNullVector(newKeyMaterial);

			// clear the new password
			if (newPsw != null){
				Zeroizer.zero(newPsw);
			}

			// change the password for the other file names:
			for (int i = 0; i < fileNames.length; i++) {	
				
				// read the encrypted content of the files 
				byte[] otherCipherBytes = ReadResources.readExternFile(fileNames[i]); 
				if (otherCipherBytes == null) {
					System.err.println("LockFrame, password change: "
							+ "Reading content of encrypted file failed: " 
							+ fileNames[i]);
					JOptionPane.showMessageDialog(PswDialogBase.getDialog().getLockFrame(), 
							PswDialogBase.getBundle().getString("cant_read_content_of_file") + ": \n"
							+ fileNames[i], 
							PswDialogBase.getBundle().getString("error"), 
							JOptionPane.ERROR_MESSAGE);
					// reset the wait cursor (modal dialog may occurred): 
					PswDialogBase.getDialog().getLockFrame().setCursor(CursorManager.getWaitCursor());
					continue;
				}

				Comparator.checkNullVector(oldKeyMaterial);
				// decrypt the encrypted content in RAM
				byte[] otherPlainBytes = CipherStuff.getInstance().decrypt(
						otherCipherBytes, 
						Arrays.copyOf(oldKeyMaterial, oldKeyMaterial.length), // use the old key to decrypt
						false );	// clear the key, do not encrypt by session key
									// (otherwise this would overwrite the new key)
				if (otherPlainBytes == null) {
					System.err.println("LockFrame, password change: "
							+ "Decrypting content of encrypted file failed: " 
							+ fileNames[i]);
					JOptionPane.showMessageDialog(PswDialogBase.getDialog().getLockFrame(), 
							PswDialogBase.getBundle().getString("decryption_failed") + ": \n"
							+ fileNames[i], 
							PswDialogBase.getBundle().getString("error"), 
							JOptionPane.ERROR_MESSAGE);
					// reset the wait cursor (modal dialog may occurred): 
					PswDialogBase.getDialog().getLockFrame().setCursor(CursorManager.getWaitCursor());
					continue;
				}

				Comparator.checkNullVector(newKeyMaterial);
				// encrypt the plain text content with the new key:
				// (can't use saveContent, because keyMaterial must not be stored)
				byte[] newKeyCipherBytes = CipherStuff.getInstance().encrypt(otherPlainBytes, 
						Arrays.copyOf(newKeyMaterial, newKeyMaterial.length), // keyMaterial is overwritten
						false ); // do not encrypt by session key: clear key
				if (newKeyCipherBytes == null) {
					System.err.println("LockFrame, password change: "
							+ "Encrypting content of file failed: " 
							+ fileNames[i]);
					JOptionPane.showMessageDialog(PswDialogBase.getDialog().getLockFrame(), 
							PswDialogBase.getBundle().getString("encryption_failed")+ " : \n"
							+ fileNames[i], 
							PswDialogBase.getBundle().getString("error"), 
							JOptionPane.ERROR_MESSAGE);
					// reset the wait cursor (modal dialog may occurred): 
					PswDialogBase.getDialog().getLockFrame().setCursor(CursorManager.getWaitCursor());
					continue;
				}

				boolean success = WriteResources.write(newKeyCipherBytes, fileNames[i], null);
				if (success == false) {
					System.err.println("LockFrame, password change: "
							+ "Writing content of file failed: " 
							+ fileNames[i]);
					JOptionPane.showMessageDialog(PswDialogBase.getDialog().getLockFrame(), 
							PswDialogBase.getBundle().getString("cant_write_content_of_file") + ": \n"
							+ fileNames[i], 
							PswDialogBase.getBundle().getString("error"), 
							JOptionPane.ERROR_MESSAGE);
					// reset the wait cursor (modal dialog may occurred): 
					PswDialogBase.getDialog().getLockFrame().setCursor(CursorManager.getWaitCursor());
					continue;
				}
			}
		} finally {
			// reset the cursor:
	    	PswDialogBase.getDialog().getLockFrame().setCursor(CursorManager.getDefaultCursor());
		}
		// zeroize the old key
		Zeroizer.zero(oldKeyMaterial);
		// encrypt and store the new key
		Comparator.checkNullVector(newKeyMaterial);
		CipherStuff.getInstance().getSessionKeyCrypt().storeKey(newKeyMaterial);
		// zeroize the key used here
		Zeroizer.zero(newKeyMaterial);
	}
	/**
	 * Change the password for one currently used content
	 * 
	 * @param plainBytes	the current content
	 * @param fileNames		the name of the file of the content
	 */
	public final void changePassword(byte[] plainBytes, String fileName){
		
		NewPasswordDialog newPswDialog = NewPasswordDialog.getInstance(this, 
				PswDialogBase.getBundle().getString("change_password"), null,
				PswDialogView.getView().getLocation());
		char[] newPsw = newPswDialog.getDialogInput();		
		
		byte[] keyMaterial = PswDialogBase.deriveKeyFromPsw(newPsw);
		if (newPsw != null){
			Zeroizer.zero(newPsw);
		}
		Comparator.checkNullVector(keyMaterial);
		// encrypt and save with new key - the new key is stored
		saveContent(keyMaterial, plainBytes, fileName);

		Zeroizer.zero(keyMaterial);
	}	


	@Override
	public void windowActivated(WindowEvent arg0) {}
	@Override
	public void windowClosed(WindowEvent arg0) {}

	@Override
	public void windowClosing(WindowEvent arg0) {
		windowClosingCommands();
		System.exit(0);
	}

	@Override
	public void windowDeactivated(WindowEvent arg0) {}
	@Override
	public void windowDeiconified(WindowEvent arg0) {}
	@Override
	public void windowIconified(WindowEvent arg0) {}
	@Override
	public void windowOpened(WindowEvent arg0) {}
}
