package cologne.eck.peafactory.peas.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Font;
import java.awt.HeadlessException;
//import java.io.File;


import java.io.File;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
//import javax.swing.filechooser.FileSystemView;

/**
 * 
 *
 * Customized JFileChooser: location is location of parent, not the middle of the screen,
 * icon is not java default icon but PEA icon. 
 * Static method to customize FileChooser (Label...). 
 */

@SuppressWarnings("serial")
public class PeaFileChooser extends JFileChooser {	

	private static File lastFile;

	public PeaFileChooser (Component comp){
		this.setLocation(comp.getLocationOnScreen());
	}

/*	public PeaFileChooser(String currentDirectoryPath) {
		super(currentDirectoryPath);
	}

	public PeaFileChooser(File currentDirectory) {
		super(currentDirectory);
	}

	public PeaFileChooser(FileSystemView fsv) {
		super(fsv);
	}

	public PeaFileChooser(File currentDirectory, FileSystemView fsv) {
		super(currentDirectory, fsv);
	}

	public PeaFileChooser(String currentDirectoryPath, FileSystemView fsv) {
		super(currentDirectoryPath, fsv);
	} */
	
    public JDialog createDialog(Component parent)
            throws HeadlessException {
    	
    	JDialog dlg = null;
    	
    	if (parent == null) {
    		dlg = super.createDialog(null);
    	} else {
        	dlg = super.createDialog(parent);
        	
            // location of parent, not middle of screen
            dlg.setLocation(parent.getLocationOnScreen());
    	}        

        // icon of PEA
        dlg.setIconImage( PswDialogView.getImage() );
        
        JPanel contentPane = (JPanel) dlg.getContentPane();
        contentPane.setBorder(PeaBorderFactory.createBorder(false));
        return dlg;
    }

    /**
     * Adds a title and message on top of the file chooser
     * 
     * @param titleText			the text of the title to set
     * @param instructionText	the text of the message to set on top of the file chooser
     * @param chooser			the file chooser to customize
     */
	public void customizeFileChooser(String titleText, String instructionText, 
			PeaFileChooser chooser) {
	    // More than customizing the first child panel is error prone
	    // because different Look&Feels build different JFileChoosers. 
	    // For example GTK+  does not work with customization of other panels
		
		if (titleText != null) {
			chooser.setDialogTitle(titleText);
		}
		
		BorderLayout layout = null;
		try {
			layout = (BorderLayout) chooser.getLayout();
		} catch (Exception e) {
			// this should not happen
			System.err.println("Wrong Layout for JFileChooser");
			return;
		}

		// get the components of the chooser
		Component[]  comps = chooser.getComponents();
		// get the component of Page_Start if available
		Component pageStartComp = null;
		Object pageStartConstraints = null;

		int compsLen = comps.length;
		for (int i = 0; i < compsLen; i++) {
			Object constraints = layout.getConstraints( comps[i]);
			//System.out.println(i + " constr: " + constraints + ",  " + comps[i].getClass());
			if (constraints.equals(BorderLayout.NORTH)
					|| constraints.equals(BorderLayout.PAGE_START)
					|| constraints.equals(BorderLayout.BEFORE_FIRST_LINE)){ 
				pageStartComp = comps[i];
				pageStartConstraints = constraints;
				break;
			} 
		}		
		
		if (instructionText != null) {
			// build a new panel, add the message 
			// and add the existing panel at page start:
			JPanel northPanel = new JPanel();
			northPanel.setLayout(new BoxLayout(northPanel, BoxLayout.Y_AXIS));
			
			JPanel messagePanel = new JPanel();			
			messagePanel.setLayout(new BoxLayout(messagePanel, BoxLayout.X_AXIS));
			
			JLabel messageLabel = new JLabel(instructionText);
			messageLabel.setFont(new Font(Font.SANS_SERIF, Font.ITALIC, 18));
			
			messagePanel.add(messageLabel);
			messagePanel.add(Box.createHorizontalStrut(10));
			
			northPanel.add(messagePanel);
			northPanel.add(Box.createVerticalStrut(20));

			if (pageStartComp != null){ 
				northPanel.add(pageStartComp);
				chooser.add(northPanel, pageStartConstraints);
			} else {
				// Look&Feel Motif
				chooser.add(northPanel, BorderLayout.PAGE_START);
			}
		}
		if (lastFile != null) {
			chooser.setSelectedFile(lastFile);
		}
		chooser.invalidate();
		chooser.validate();
		chooser.repaint();
	}
	/**
	 * Remember the selected file to display next time:
	 */
	@Override
    public void approveSelection() {
        lastFile = getSelectedFile();
        super.approveSelection();
    }
	
    // test:
/*	public static void main(String[] args) {
		PeaFileChooser pfc = new PeaFileChooser();
		pfc.setVisible(true);
	} */

}
