package cologne.eck.peafactory.peas.image_pea;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Generates the image pea. 
 */

import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import cologne.eck.peafactory.PeaFactory;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.gui.*;
import cologne.eck.peafactory.peagen.DataType;
import cologne.eck.peafactory.peagen.JarStuff;
import cologne.eck.peafactory.tools.ReadResources;
import cologne.eck.peafactory.tools.WriteResources;

//import cologne.eck.peafactory.tools.Help;


public final class ImageType extends DataType {
	
	// default values:
	private static byte[] imageBytes = null;
	
	private static String imageName = null;

	// returns null if not a valid image
	public final Image getImageFromInternalImageName	() {
		
		if (PeaFactory.isBlankPea() == true) {
			return null;
		}
		
		// Maximal width and height to display in MainView:
		int picWidth = 500;
		int picHeight = 500;				
		
		// get image
		 BufferedImage image = null;
		 try {
			image = ImageIO.read(new File(imageName));
		} catch (IOException e) {
			System.err.println("image read failed");
			return null;
		}	 
		 
		 if (image == null) {
			 System.err.println("ImageType: image null, imageName: " + imageName + ". Invalid image? ");
			 //System.exit(3);
			 return null;
		 }

		 int w = picWidth,h = picHeight;	

		 // resize image to size from menu (or default)
		 double op = 0;		 
	 
		 if (image.getWidth() < picWidth) {
			 op = (double)picWidth / image.getWidth();
			 if (image.getHeight() * op > picHeight) op = (double)picHeight / image.getHeight();
			 w = (int) (image.getWidth() * op);
			 h = (int) (image.getHeight() * op);
		 } else {
			 op = image.getWidth() / (double)picWidth;
			 if ( (image.getHeight() / op) > picHeight) op = image.getHeight() / (double)picHeight;
			 w = (int) (image.getWidth() / op);
			 h = (int) (image.getHeight() / op);
		 }		 
		 
		 Image scaledImage = image.getScaledInstance(w, h, Image.SCALE_FAST);
		 
		 return scaledImage;
	}
	
	//===============================================
	// Getter & Setter
	
	public static final String getImageName() {
		return imageName;
	}
	public static final void setImageName(String _imageName) {
		imageName = _imageName;
	}

	
	@Override
	public void processData(byte[] keyMaterial) {
		
		// get plainBytes: 
		// if (MainView.getOpenedFileName() == null) from imageBytes (set before ImageTypePanel)
		//     (the fileName to store was set as imageName not as MainView.getOpenedFileName() )
		// else from file MainView.getOpenedFileName() (and store cipherBytes there later)
		
		byte[] plainBytes = null;

		if (PeaFactory.isBlankPea() ==  true) {
			// create directory to contain jar file and start files
			String dirName = JarStuff.getJarFileName().substring(0, (JarStuff.getJarFileName().length() - 4)); // extract ".jar"
			new File(dirName).mkdirs();

			// do not encrypt anything
			return;
			
		} else { // not blank PEA		
			
			if (MainView.getOpenedFileName() == null) {
				plainBytes = imageBytes;

			} else {
				plainBytes = ReadResources.readExternFile(MainView.getOpenedFileName());
			}
			// check plainBytes
			if (plainBytes == null) { // read-write was tested in ImageTypePanel, so this might not happen
				System.err.println("ImageType: getting plainBytes failed");
				return;
			} 
		}			

		//Help.printBytes("ImageType keyMaterial",  keyMaterial);		
		byte[] cipherBytes = CipherStuff.getInstance().encrypt(plainBytes, keyMaterial, false);
		//Help.printlastBytes("ImageType last 128 cipherBytes", cipherBytes, 128);
		
		// store cipherBytes + settings:
		if (MainView.getOpenedFileName() == null) {
			
			// write text.lock file in directory resources of program (not pea!)
			// and add it later in CreateJarFile to the jar
			WriteResources.write(cipherBytes, "resources/text.lock", null);
						
		} else { // write cipherBytes in external file and file path in text.lock			
			WriteResources.write(cipherBytes, MainView.getOpenedFileName(), null);
		}		
	}
	

	@Override
	public String getDescription() {
		return PeaFactory.getLanguagesBundle().getString("image_description");
	}

	@Override
	public String getTypeName() {
		return "image";
	}

	@Override
	public byte[] getData() {
		return imageBytes;
	}

	@Override
	public void setData(byte[] data) {
		imageBytes = data;		
	}
}
