package cologne.eck.peafactory.peas.image_pea;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Main view of image pea. 
 */

/*
 * Frame to display images, decrypted in PswDialogImage
 * (this is not a daughter class of LockFrame!)
 * 
 */



import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.awt.image.ImageObserver;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;

import javax.activation.MimetypesFileTypeMap;
import javax.imageio.ImageIO;
import javax.swing.*;

import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.peas.PswDialogBase;
import cologne.eck.peafactory.peas.gui.CursorManager;
import cologne.eck.peafactory.peas.gui.HelpMenu;
import cologne.eck.peafactory.peas.gui.MultiFileSelectionDialog;
import cologne.eck.peafactory.peas.gui.LockFrame;
import cologne.eck.peafactory.peas.gui.PeaMenuBar;
import cologne.eck.peafactory.peas.gui.PswDialogView;
import cologne.eck.peafactory.peas.gui.SingleFileSelectionDialog;
import cologne.eck.peafactory.tools.ReadResources;
import cologne.eck.peafactory.tools.WriteResources;
import cologne.eck.peafactory.tools.Zeroizer;

@SuppressWarnings("serial")
final class LockFrameImage extends LockFrame 
	implements ActionListener, ImageObserver {	
	//, AdjustmentListener {
	
	/**
	 * All file names of the image folder
	 */
	private static ArrayList <String> fileList;
	
	/**
	 * The index of the current image
	 */
	int index = 0; 
	
	// the loaded image as an array of bytes
	private static byte[] imageBytes;
	
	// check if there is already an instance
	private static boolean isInstantiated = false;
	
	// the image to show
	private static BufferedImage image = null;		
	// scroll pane containing the image
	private JScrollPane scrollPane = null;
	// vertical ScrollBar
	private JScrollBar verticalBar = null;//scrollPane.getVerticalScrollBar();
	// the label containing the scroll pane
	private JLabel picLabel = null;
	// label to show error and info messages 
	private JLabel messageLabel = null;


	// zoom value for width: additional width
	private static int zoomWidth = 0;
	// zoom value for height: additional height
	private static int zoomHeight = 0;
	
	// resized default width from PswDialogImage
	private int imgWidth = 0;
	// resized default height from PswDialogImage
	private int imgHeight = 0;

	
	private LockFrameImage(Point location) {
		
	    this.setIconImage(PswDialogView.getImage());
		
		this.addWindowListener(this);// for windowClosing
		
		JPanel contentPane = new JPanel();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
		this.setContentPane(contentPane);		
		
		JPanel menuPanel = new JPanel();
		menuPanel.setLayout(new BoxLayout(menuPanel, BoxLayout.X_AXIS));
		menuPanel.setMaximumSize(new Dimension(600, 30));
//		menuPanel.setMinimumSize(new Dimension(600, 30));
		
		PeaMenuBar menuBar = new PeaMenuBar();
		//menuBar.setBorder(new EtchedBorder() );
		
		JMenu menu = new JMenu();
		menu.setText(PswDialogBase.getBundle().getString("file"));//"Menu");
		menu.setMnemonic(KeyEvent.VK_M);
		menuBar.add(menu);
		
		JMenuItem encryptItem = new JMenuItem();
		encryptItem.setText(PswDialogBase.getBundle().getString("close_encrypted"));
		encryptItem.setActionCommand("encrypted");
		encryptItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		encryptItem.setMnemonic(KeyEvent.VK_E);
		encryptItem.addActionListener(this);
		menu.add(encryptItem);

		
		// if image is inside the jar, there is no possibility to chnage the
		// password or to leave the image unencrypted
		if (PswDialogBase.isInsideJar() == false) {
			JMenuItem unencryptItem = new JMenuItem();
			unencryptItem.setText(PswDialogBase.getBundle().getString("close_unencrypted"));
			unencryptItem.setActionCommand("unencrypted");
			unencryptItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
			unencryptItem.setMnemonic(KeyEvent.VK_U);
			unencryptItem.addActionListener(this);
			menu.add(unencryptItem);
			
			JMenuItem pswChangeItem = new JMenuItem();
			pswChangeItem.setText(PswDialogBase.getBundle().getString("change_password"));//"change password");
			pswChangeItem.setActionCommand("changePassword");
			pswChangeItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
			pswChangeItem.setMnemonic(KeyEvent.VK_P);
			pswChangeItem.addActionListener(this);
			menu.add(pswChangeItem);
		}
		
		JMenuItem gotoItem = new JMenuItem();
		gotoItem.setText(PswDialogBase.getBundle().getString("go_to"));
		gotoItem.setActionCommand("goto");
		gotoItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		gotoItem.setMnemonic(KeyEvent.VK_G);
		gotoItem.addActionListener(this);
		menu.add(gotoItem);
		

		JMenu helpMenu = new HelpMenu();
		menuBar.add(helpMenu);

		menuPanel.add(menuBar);
		menuPanel.add(Box.createHorizontalGlue());
		contentPane.add(menuPanel);
		
		JPanel messagePanel = new JPanel();
		messagePanel.setLayout(new BoxLayout(messagePanel, BoxLayout.X_AXIS));
//		messagePanel.setPreferredSize(new Dimension (300, 30));
		messagePanel.setMaximumSize(new Dimension(
				(int)PswDialogView.getScreenwidth()-100, 50));
		messagePanel.setMinimumSize(new Dimension (300, 30));
		
		JButton previousButton = new JButton();//previous");
		Image previousImage = null;
		URL url = this.getClass().getResource("resources/zurueck.png");
		if (url == null) {
			try{
				previousImage = new ImageIcon(getClass().getClassLoader().getResource("resources/zurueck.png")).getImage();
			} catch (Exception e) {
				//System.out.println("image not found " + e.getMessage() + ", " + e.toString());
			}
		} else {			
			previousImage = new ImageIcon(url).getImage();
		}
		if (previousImage != null) {
			previousButton.setIcon(new ImageIcon(previousImage));
		} else { // set characters
			previousButton.setText(" <-");
			previousButton.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 15));
		}
		previousButton.setMaximumSize(new Dimension(75,25));
		previousButton.setMargin(new Insets(1,1,1,1));
		previousButton.addActionListener(this);
		previousButton.setActionCommand( "previous");
		
		messagePanel.add(previousButton);
		messagePanel.add(Box.createHorizontalStrut(25));
		

		messageLabel = new JLabel(" Image PEA " + PswDialogBase.getDialog().getVersion());
		//messageLabel.setPreferredSize(new Dimension(300, 30));
		messageLabel.setMinimumSize(new Dimension(100, 30));
		messageLabel.setMaximumSize(new Dimension((int)PswDialogView.getScreenwidth() - 250, 30));
		messagePanel.add(messageLabel);
		messagePanel.add(Box.createHorizontalGlue());		
		
		JButton nextButton = new JButton();
		Image nextImage = null;
		url = this.getClass().getResource("resources/weiter.png");
		if (url == null) {
			try{
				nextImage = new ImageIcon(getClass().getClassLoader().getResource("resources/weiter.png")).getImage();
			} catch (Exception e) {
				//System.out.println("image not found " + e.getMessage() + ", " + e.toString());
			}
		} else {			
			nextImage = new ImageIcon(url).getImage();
		}

		if (nextImage != null) {
			nextButton.setIcon(new ImageIcon(nextImage));
		} else { // set characters
			nextButton.setText("-> ");
			nextButton.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 15));
		}		
		//JButton nextButton = new JButton("next");
		nextButton.setMaximumSize(new Dimension(75,25));
		nextButton.setMinimumSize(new Dimension(75,25));
		nextButton.setMargin(new Insets(1,1,1,1));
		nextButton.addActionListener(this);
		nextButton.setActionCommand( "next");
		messagePanel.add(nextButton);
		
		contentPane.add(messagePanel);
		
		boolean success = setNewImage(imageBytes);
		Window win;
		if (PswDialogView.getView().isVisible()){
			win = PswDialogView.getView();
		} else {
			win = PswDialogBase.getDialog().getLockFrame();
		}
		if (success == false) {
			
			String decryptionMessage = "";
		
			if(CipherStuff.getErrorMessage() != null){
				decryptionMessage = PswDialogBase.getBundle().getString(CipherStuff.getErrorMessage());
			}
			CipherStuff.setErrorMessage(null);
			JOptionPane.showMessageDialog(win, 
					PswDialogBase.getBundle().getString("cant_load_image")
					+ "\n   " + fileList.get(index)
					+ "\n" + decryptionMessage, 
					PswDialogBase.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);

			int len = fileList.size();
			if (len <= 1) {
				JOptionPane.showMessageDialog(win, 
						PswDialogBase.getBundle().getString("cant_load_any_image"), 
						PswDialogBase.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
				System.exit(0);
			} else {
				index++;
				while (success == false && index < len){
					success = loadOtherImage(index);	
					index++;
				}
			}
		}
		if (success == false) {
			JOptionPane.showMessageDialog(win, 
					PswDialogBase.getBundle().getString("cant_load_any_image")
					+ "\n  " + PswDialogBase.getBundle().getString("remove_and_try_other"), 
					PswDialogBase.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
			System.exit(0);
		}
				
		// set maximum size, depending on screen size 
		scrollPane.setMaximumSize(new Dimension (
						(int)PswDialogView.getScreenwidth() - 100, 
						(int) PswDialogView.getScreenheight() - 200 ));// think of buttons
		// avoid page scroll for PDFs
		verticalBar = scrollPane.getVerticalScrollBar();
		verticalBar.setValue(5);

		// register scroll to begin and to end to render next pages
//		verticalBar.addAdjustmentListener(this);
		contentPane.add(scrollPane);
		
		JPanel zoomPanel = new JPanel();
		zoomPanel.setMaximumSize(new Dimension(400, 30));
		zoomPanel.setMinimumSize(new Dimension(200, 30));
		
		JButton zoomInButton = new JButton ("+");
		zoomInButton.setActionCommand("zoomIn");
		zoomInButton.addActionListener(this);
		zoomPanel.add(zoomInButton);	
		
		JButton zoomOutButton = new JButton ("-");
		zoomOutButton.setActionCommand("zoomOut");
		zoomOutButton.addActionListener(this);
		zoomPanel.add(zoomOutButton);	
		
		zoomPanel.add(zoomInButton);
		zoomPanel.add(zoomOutButton);
		
		contentPane.add(zoomPanel);		

		this.setLocation(location);
		
		// pack or setMaximumSize/MinimumSize depending on screen size and image size
		if (this.getPreferredSize().getWidth() > PswDialogView.getScreenwidth() - 100
			|| this.getPreferredSize().getHeight() > PswDialogView.getScreenheight() - 100) {
			
			this.setMaximumSize(
					new Dimension (
							(int)PswDialogView.getScreenwidth() - 100,
							(int)PswDialogView.getScreenheight() - 100));
			int w = PswDialogImage.getImageWidth();
			if (w > PswDialogView.getScreenwidth() - 100){
				w = (int) (PswDialogView.getScreenwidth() - 100);
			}
			int h = PswDialogImage.getImageHeight();
			if (h > PswDialogView.getScreenheight() - 100){
				h = (int) (PswDialogView.getScreenheight() - 100);
			}
			// is is the shown size if the screen is large enough
			this.setMinimumSize(new Dimension (w, h));			
		} else {
			
			pack();
		}
		isInstantiated = true;
	}
	/**
	 * Returns an already instantiated instance 
	 * 
	 * @return 
	 */
	protected final static LockFrameImage getInstance(Point loc) {
		LockFrameImage frame = null;
		if (isInstantiated == false) {
			frame = new LockFrameImage(loc);
		} else {
			//return null
		}		
		return frame;
	}
	 // just to test the view	 
	public static void main(String[] args) {

		PswDialogView.setUI();
		/* ********************* later version... **************	
		File pdfFile = new File("resources" + File.separator + "catena-v3.3.pdf");
		PdfToPngConverter.setDocument(pdfFile);
		PdfToPngConverter.convert(page, page + pageNumber);		

		LockFrameImage.setImageBytes(ReadResources.readExternFile(pdfFile.getPath().substring(0,  pdfFile.getPath().length() -3) + "png"));
		*/
//		File testFile = new File("resources/Phahlbaudorf.JPG");
		File testFile = new File("resources/images/drop.png");
		fileList = new ArrayList<String>();
		String imageFolder = "resources/images";
		File folder = new File(imageFolder);
		File[] files = folder.listFiles();
		int len = files.length;
		for(int i = 0; i < len; i++) {
			fileList.add(files[i].getAbsolutePath());
			System.out.println(files[i].getAbsolutePath());
		}
		LockFrameImage.setImageBytes(ReadResources.readExternFile(testFile.getAbsolutePath()));

		LockFrameImage frame = new LockFrameImage(new Point(100,100));
		frame.messageLabel.setText(testFile.getAbsolutePath());
		
		frame.setVisible(true);
	} 
	
	/**
	 * The method resizes an image to keep the ratio of width and height
	 * to avoid distortions of the image. The result width is never smaller
	 * than the default width / 4. 
	 * 
	 * @param image		the image to resize
	 * @param width		the width of the frame to display the image
	 * @param height	the height of the frame to display the image
	 * 
	 * @return			resulting width ad height, stored in an array:
	 * 					result[0] is the new width, result[1] the new height to display
	 */
	private final int[] resizeImage(BufferedImage image, int width, int height){
		// keep ratio of image width and height: resize the image 

		int[] result = new int[2];
		int displayedWidth = width;
		int displayedHeight = height;			
		int imageWidth = image.getWidth();
		int imageHeight = image.getHeight();
		double op = 0; // factor to resize		
		//System.out.println("image w: " + image.getWidth() + ", h: " + image.getHeight()
		//		+ "\n" + "default w: " + displayedWidth + ", h: " + displayedHeight);
		 
		 if (imageWidth < width) {
			 		
			 op = (double)width / imageWidth;

			 if (imageHeight * op > height) {
				 
				 op = (double)height / imageHeight;
				 // do not resize smaller than half of default width: 
				 // this is normally a rendered PDF
				 if ( (imageWidth * op) < (width / 4) ) {
					 op = 1;//imageWidth / (double)width;
				 }
			 }
			 
			 displayedWidth = (int) (imageWidth * op);
			 displayedHeight = (int) (imageHeight * op);
		 } else {
			 op = imageWidth / (double)width;

			 if ( (imageHeight / op) > height) {
				 op = imageHeight / (double)height;
				 // do not resize smaller than half of default width: 
				 // this is normally a rendered PDF
				 if ( (imageWidth / op) < (width / 4) ) {
					 op = 1;//imageWidth / (double)width;
				 }
			 }
			 
			 displayedWidth = (int) (imageWidth / op);
			 displayedHeight = (int) (imageHeight / op);
		 }	
		 result[0] = displayedWidth;
		 result[1] = displayedHeight;
		 //System.out.println("result w: " + result[0] + ", h: " + result[1]);
		 return result;
	}

	/**
	 * Zoom in or out: Resize the image, display the image and set the scroll bars 
	 * 
	 * @param in	if true: zoom in, otherwise zoom out
	 */
	private final void zoom(boolean in){
		
		  int previouseV = scrollPane.getVerticalScrollBar().getValue();
		  int previouseH = scrollPane.getHorizontalScrollBar().getValue();
		
		if (in == true) {			

			zoomWidth += imgWidth/2;
			zoomHeight += imgHeight/2;
			
		} else {
			
			zoomWidth -= imgWidth/2;
			zoomHeight -= imgHeight/2;	
			
			if (imgWidth + zoomWidth <= 0 || imgHeight + zoomHeight <= 0){
				//System.out.println("Zoom out: invalid size values for image.");				
				zoomWidth += imgWidth/2;
				zoomHeight += imgHeight/2;
				// avoid IllegalArgumentException: do not paint the image
				return;
			}
		}
		try {
			// new compressed/resized image
			BufferedImage  compressedImage = new BufferedImage(
					imgWidth + zoomWidth, imgHeight + zoomHeight,
                  BufferedImage.TYPE_INT_ARGB);
            Graphics2D g = compressedImage.createGraphics();
            g.drawImage(image, 0, 0, imgWidth + zoomWidth, imgHeight + zoomHeight, this);              
            g.dispose();		
		  
             
            picLabel.setIcon(new ImageIcon(compressedImage));
               
      		if (in == true) {      	
                scrollPane.getVerticalScrollBar().setValue((int) (previouseV + imgHeight/4));
                scrollPane.getHorizontalScrollBar().setValue((int) (previouseH + imgWidth/4));                
      		} else {
                scrollPane.getVerticalScrollBar().setValue((int) (previouseV - imgHeight/4));
                scrollPane.getHorizontalScrollBar().setValue((int) (previouseH - imgWidth/4));
      		}
		} catch (IllegalArgumentException e){  // width and height are < 0

			if (PswDialogBase.getWorkingMode().equals("-t")) { // test mode
				System.out.println(e.toString() + ", " + e.getMessage());
			}		 
			// reset the zoom
			if (in == true) {
				 zoomWidth -= PswDialogImage.getImageWidth() / 2;
				 zoomHeight -= PswDialogImage.getImageHeight() / 2;
			} else {
				 zoomWidth += PswDialogImage.getImageWidth() / 2;
				 zoomHeight += PswDialogImage.getImageHeight() / 2;		
			}
		} catch (OutOfMemoryError ex) { // ... if the image is too large
			if (PswDialogBase.getWorkingMode().equals("-t")) { // test mode
				System.err.println(ex.toString() + " - " + ex.getMessage());
			  
				System.err.println("total memory: " + Runtime.getRuntime().totalMemory());
				System.err.println("free memory: " + Runtime.getRuntime().freeMemory());
				// used space: image width * image height * 8 (TYPE_INT_ARGB) 
				System.err.println("used space for image: " + ((imgWidth + zoomWidth) * (imgHeight + zoomHeight) * 8));
			}		  
			zoomWidth -= imgWidth/2;
			zoomHeight -= imgHeight/2;
		}
	}
	/**
	 * Checks if a file is a valid image that can be shown
	 * by ImagePEA. The file must exist and must have read access,
	 * otherwise the check fails. The file is at least checked by 
	 * ImageIO, but before - depending on parameters - checked by
	 * it's MIME type ("image") and it's extension (PNG, GIF, BMP, JEPG, JPG) 
	 * 
	 * @param file				the file to check
	 * @param checkExtension	whether to check the extension of the file or not
	 * @param checkMimeType		whether to check the MIME type of the file or not
	 * 
	 * @return	true if the file is a image that can be shown by the Image PEA
	 */
	public static boolean checkImage(File file, boolean checkExtension, boolean checkMimeType) {
		
		String fileName = file.getAbsolutePath();
		
		// check MIME type:
		if (checkMimeType == true) {
			 String mimetype= new MimetypesFileTypeMap().getContentType(file);
		        String type = mimetype.split("/")[0];
		        if( ! type.toLowerCase().equals("image")) {
		        	return false;
		        }
		}
		
		// check extension: 
		if (checkExtension == true) {
			String extension = fileName.substring(fileName.lastIndexOf('.') + 1,
				fileName.length()).toUpperCase();
			//System.out.println("extension: " + extension);
			if (! (extension.equals("PNG") || extension.equals("GIF")
					|| extension.equals("BMP") || extension.equals("JPG")
					|| extension.equals("JPEG")) ){ 
				return false;
			}
		}
		
		// check by ImageIO:
		try{
			BufferedImage bi = ImageIO.read(file);	
			if (bi == null) {
				return false;
			}
		} catch (Exception e) {
			return false;
		}
		return true;
	}

	/**
	 * Set the image to show. This class holds a copy of the image as an array of bytes
	 * 
	 * @param input		the image as an array of bytes
	 */
	protected final static void setImageBytes( byte[] input) {//, String fileName ) {
		imageBytes = new byte[input.length];
		System.arraycopy(input,  0,  imageBytes,  0,  input.length);
		//currentFileName = fileName;
	}
	
	private final boolean setNewImage(byte[]imageBytes) { //, String fileName) {

		// get image from file name:		
		try {					
			image = ImageIO.read(new ByteArrayInputStream(imageBytes));

		} catch (IOException e) {			
			System.err.println("LockFrameImage: " + e.toString() + " - " + e.getMessage());
			if (PswDialogBase.getWorkingMode().equals("-t")) { // test mode				
				e.printStackTrace();
			} 
		}
		Window win;
		if (PswDialogView.getView().isVisible()){
			win = PswDialogView.getView();
		} else {
			win = PswDialogBase.getDialog().getLockFrame();
		}
		if (image ==  null) { // invalid file
			JOptionPane.showMessageDialog(win, 
					PswDialogBase.getBundle().getString("invalid_image_file")//"Invalid or unsuitable image file: \n"                      
					      + PswDialogImage.getEncryptedFileName(), 
					      PswDialogBase.getBundle().getString("error"),                                
					   JOptionPane.ERROR_MESSAGE);   
			return false;
		}
		// resize the image
		int[] newSize = resizeImage(image, PswDialogImage.getImageWidth(), PswDialogImage.getImageHeight());
		imgWidth = newSize[0];
		imgHeight = newSize[1];	
		
		// check if image is larger than default values for width and height: 
		if (image.getWidth() > imgWidth || image.getHeight() > imgHeight){
			// compress the image:
			BufferedImage  resizedImage = new BufferedImage(imgWidth + zoomWidth, imgHeight + zoomHeight,
	        BufferedImage.TYPE_INT_ARGB);
	        Graphics2D g = resizedImage.createGraphics();
	        g.drawImage(image, 0, 0, imgWidth + zoomWidth, imgHeight + zoomHeight, this);              
	        g.dispose();	

	        if (picLabel== null) {
	        	picLabel= new JLabel();
	        }
			picLabel.removeAll();
			picLabel.setIcon(new ImageIcon(resizedImage));
//			scrollPane = new JScrollPane(picLabel);
			if (scrollPane == null) {
				scrollPane = new JScrollPane(picLabel);
			}
			scrollPane.setPreferredSize(new Dimension(imgWidth, imgHeight));
		} else {
			// do not compress
	        if (picLabel== null) {
	        	picLabel= new JLabel();
	        }
			picLabel.removeAll();
			picLabel.setIcon(new ImageIcon(image));

			if (scrollPane == null) {
				scrollPane = new JScrollPane(picLabel);
			}
			scrollPane.setPreferredSize(new Dimension(imgWidth, imgHeight));
			imgWidth = image.getWidth();
			imgHeight = image.getHeight();
		}
		//currentFileName = fileName;
		
		this.pack();
		return true;
	}
	
	public void setMessage(String _message) {

		messageLabel.setText(_message);
	}

	@Override
	public void actionPerformed(ActionEvent ape) {

		String command = ape.getActionCommand();

		if (command.equals("unencrypted")) {
			
			MultiFileSelectionDialog dialog = new MultiFileSelectionDialog(
					this, this.getLocationOnScreen(), 
					PswDialogBase.getBundle().getString("select_files_remain_unencrypted"),
					fileList);
			ArrayList<String> unencryptedList = MultiFileSelectionDialog.getSelectedFiles();
			dialog.dispose();
			
			if (unencryptedList == null || unencryptedList.size() == 0) {
				return;
			}
			try {
				// set wait cursor
				this.setCursor(CursorManager.getWaitCursor());
				// the current displayed file:
				// the index will change because elements will be removed
				String currentFileName = fileList.get(index);
				
				if ( (unencryptedList != null) 
						&& (unencryptedList.size() > 0) ) {
					int len = unencryptedList.size();
					for (int i = 0; i < len; i++) {
						
						if (unencryptedList.get(i) == null) {
							System.err.println("File name is null at index " + i);
						}
						
						if (unencryptedList.get(i).equals(currentFileName)) { // current shown image:
							// close at least
							continue;
						}
						//read cipherBytes, decrypt and store
						byte[] cipherBytes = ReadResources.readExternFile(unencryptedList.get(i));
						byte[] plainBytes = CipherStuff.getInstance().decrypt(cipherBytes, null, true );
						if (plainBytes == null) {
							System.err.println("Decryption failed at index " + i
									+ " for file name " + unencryptedList.get(i));
							JOptionPane.showMessageDialog(PswDialogBase.getDialog().getLockFrame(), 
									PswDialogBase.getBundle().getString("decryption_failed")
									+ "\n" + unencryptedList.get(i)
									+ PswDialogBase.getBundle().getString(CipherStuff.getErrorMessage()), 
									PswDialogBase.getBundle().getString("error"), 
									JOptionPane.ERROR_MESSAGE);
							CipherStuff.setErrorMessage(null);
							continue;
						}
						WriteResources.write(plainBytes, unencryptedList.get(i), null );

						// set null to mark as invalid
						fileList.set(i, null);// unencryptedList.get(i) );
					}
					if (fileList.contains(currentFileName) ) {
						//read cipherBytes, decrypt and store
						byte[] cipherBytes = ReadResources.readExternFile(currentFileName);
						byte[] plainBytes = CipherStuff.getInstance().decrypt(cipherBytes, null, true );
						if (plainBytes == null) {
							System.err.println("Decryption failed for " + currentFileName);
							JOptionPane.showMessageDialog(PswDialogBase.getDialog().getLockFrame(), 
									PswDialogBase.getBundle().getString("decryption_failed")
									+ "\n" + currentFileName
									+ PswDialogBase.getBundle().getString(CipherStuff.getErrorMessage()),
									PswDialogBase.getBundle().getString("error"), 
									JOptionPane.ERROR_MESSAGE);
							CipherStuff.setErrorMessage(null);
						}
						WriteResources.write(plainBytes, currentFileName, null );
						// give message:
						JOptionPane.showMessageDialog(this,
								PswDialogBase.getBundle().getString("current_file_closed"));
						// close program: 
						Zeroizer.zero(imageBytes);
						System.exit(0);
					}
				}


			} finally {
				this.setCursor(CursorManager.getDefaultCursor());
			}
			// only close program if current file was selected
			//System.exit(0);
			
		} else if (command.equals("encrypted")){
			// file was not modified, just loaded unencrypted in RAM
			PswDialogBase.pathFileCheck(fileList);
			Zeroizer.zero(imageBytes);
			System.exit(0);
		} else if (command.equals("changePassword")){
		
			// show a list of files
			MultiFileSelectionDialog dialog = new MultiFileSelectionDialog(
					this, this.getLocationOnScreen(), 
					PswDialogBase.getBundle().getString("Select_file_for_password_change"),
					fileList);
			
			// the list of the selected files
			ArrayList<String> unencryptedList = MultiFileSelectionDialog.getSelectedFiles();
			// check if current file is included:
			if ( ! (unencryptedList.contains(fileList.get(index))) ){
				JOptionPane.showMessageDialog(this, 
						PswDialogBase.getBundle().getString("must_contain_current_file") +":\n" + fileList.get(index), 
						PswDialogBase.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
				return;
			} 
			dialog.dispose();			

			// Note: Wait cursor is set in LockFrame.changePassword
			String[] arrayToChangePassword = null;
			if (unencryptedList != null && unencryptedList.size() > 0) {
				// create array from the list:
				arrayToChangePassword = new String[unencryptedList.size()];
				arrayToChangePassword = unencryptedList.toArray(arrayToChangePassword);
			}
			changePassword( arrayToChangePassword);	// file names to change

		} else if (command.equals("zoomIn")){
			zoom(true);
		} else if (command.equals("zoomOut")){
			zoom(false);
		} else if (command.equals("about")) {
			PswDialogView.showAboutDialog("Image Lock PEA", PswDialogBase.getBundle().getString("supported_formats") + ": png, jpg, gif, wbmp", this);
			
		} else if (command.equals("infos")){			
			
			PswDialogView.showInfoDialog(this);
		} else if (command.equals("previous")){				
			
			int newIndex = 0;
			if (index == 0) { // begin...
				// show last image:
				newIndex = fileList.size() - 1;
			} else {
				// show previous image:
				newIndex = index - 1;
			}
			boolean success = false;
			
			try {
				// set wait cursor
				this.setCursor(CursorManager.getWaitCursor());			
			
				while (newIndex != index) { // index is only updated for success
					// end of loop if no valid image is found
	
					if (fileList.get(newIndex) == null) {
						// was marked as invalid
					} else {
						// try to load previous image:
						success = loadOtherImage(newIndex);
					}					
					if (success == true) {
						this.setCursor(CursorManager.getDefaultCursor());
						return;
					} else {
						JOptionPane.showMessageDialog(this, 
								PswDialogBase.getBundle().getString("cant_load_image")
								+ ": \n" + fileList.get(newIndex)
								+ "\n" + PswDialogBase.getBundle().getString(CipherStuff.getErrorMessage()),
								PswDialogBase.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
						// reset wait cursor because of modal dialog
						this.setCursor(CursorManager.getWaitCursor());
						CipherStuff.setErrorMessage(null);
						// try image before previous image
						newIndex--;
						if (newIndex < 0) { // begin...
							// show last image:
							newIndex = fileList.size() - 1;
						}
					}
				}
				if (success == false) {
					
					// try to load current image again:
					success = loadOtherImage(index);
					if (success = false) {
						JOptionPane.showMessageDialog(this, 
								PswDialogBase.getBundle().getString("cant_load_any_image")
								+ "\n" + PswDialogBase.getBundle().getString(CipherStuff.getErrorMessage()),
								PswDialogBase.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
						System.exit(0);
					}
				}
			} finally {
				this.setCursor(CursorManager.getDefaultCursor());
			}
			
		} else if (command.equals("next")){		
			
			// register loading and decryption sucess of new image
			boolean success = false;
			// index of the image to show next
			int newIndex = 0;
			if (index + 1 >= fileList.size()) { // end...
		    	// set first image:
		    	newIndex = 0;
			} else {
				// set next image:
				newIndex = index + 1;
			}
			try {
				// set wait cursor
				this.setCursor(CursorManager.getWaitCursor());
						
				while (newIndex != index) { // index is only updated for success
					// end of loop if no valid image is found
			
					if (fileList.get(newIndex) == null) {
						// image is marked as invalid...
					} else {					
						// try to load next image:
						success = loadOtherImage(newIndex);
					}	
					
					if (success == true) {
						this.setCursor(CursorManager.getDefaultCursor());
						return;
					} else {
						JOptionPane.showMessageDialog(this, 
								PswDialogBase.getBundle().getString("cant_load_image") + ": \n" + fileList.get(newIndex)
								+ "\n" + PswDialogBase.getBundle().getString(CipherStuff.getErrorMessage()),
								PswDialogBase.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
						CipherStuff.setErrorMessage(null);
					}
					// reset wait cursor because of modal dialog
					this.setCursor(CursorManager.getWaitCursor());
					
					// try image after next image
					newIndex++;
					if (newIndex >= fileList.size()) { // end...
				    	// set first image:
				    	newIndex = 0;
				    	if (newIndex == index) {
				    		// no other valid image found
							JOptionPane.showMessageDialog(this, 
									PswDialogBase.getBundle().getString("cant_load_other_image"), 
									PswDialogBase.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
							// reset wait cursor because of modal dialog
							this.setCursor(CursorManager.getWaitCursor());
				    	}
					}			
				} // end of loop
				if (success == false) {
					// try to load current image again:
					success = loadOtherImage(index);
					if (success = false) {
						JOptionPane.showMessageDialog(this, 
								PswDialogBase.getBundle().getString("cant_load_any_image")
								+ "\n" + PswDialogBase.getBundle().getString(CipherStuff.getErrorMessage()),
								PswDialogBase.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
						System.exit(0);
					}
				}
			} finally {
				this.setCursor(CursorManager.getDefaultCursor());
			}
		}  else if (command.equals("goto")) {
			SingleFileSelectionDialog dialog = new SingleFileSelectionDialog(
					PswDialogBase.getDialog().getLockFrame(), 
					PswDialogBase.getDialog().getLockFrame().getLocationOnScreen(), 
					PswDialogBase.getBundle().getString("select_image_file"), 
					fileList);
			String fileName = SingleFileSelectionDialog.getSelectedFileName();
			dialog.dispose();
			boolean success = loadOtherImage(fileList.indexOf(fileName));
			if (success == false) {
				JOptionPane.showMessageDialog(this, 
						PswDialogBase.getBundle().getString("cant_load_image") + ": \n" + fileList.get(fileList.indexOf(fileName))
						+ "\n"+ PswDialogBase.getBundle().getString(CipherStuff.getErrorMessage()), 
						PswDialogBase.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
				CipherStuff.setErrorMessage(null);
				// load last image:
				success = loadOtherImage(index);
				if (success == false) {
					JOptionPane.showMessageDialog(this, 
							PswDialogBase.getBundle().getString("cant_load_any_image"), 
							PswDialogBase.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
					System.exit(0);
				}
			} else {
				messageLabel.setText(fileName);				
			}
		}
	}

	private boolean loadOtherImage(int newIndex) {
		
		boolean success = true;
		byte[] newImageBytes = null;

			
		try {
			byte[] cipherBytes = ReadResources.readExternFile(fileList.get(newIndex));
			//currentFileName = fileList.get(fileList.size() - 1);
			newImageBytes = CipherStuff.getInstance().decrypt(cipherBytes, null, true );
		
		} catch (Exception e) {
			// show the previous image:
			System.err.println(e.toString() + ", " + e.getMessage());
			success = false;
			// set element null to indicate error
			fileList.set(newIndex, null);	
		}
		if (newImageBytes != null) {
			// set newImageByte as current image
			imageBytes = newImageBytes;
			// load new image:
			boolean loadSuccess = setNewImage(imageBytes);
			if (loadSuccess == false){
				return false;
			}
			messageLabel.setText(fileList.get(newIndex));
			// set new index as current index
			index = newIndex;
		} else {
			System.err.println("An error occured");
							
			success = false;
		}		
		return success;
	}

	@Override
	protected void windowClosingCommands() {
		// ask to remember files:
		PswDialogBase.pathFileCheck(fileList);
				
		CipherStuff.getInstance().getSessionKeyCrypt().clearKeys();
		Zeroizer.zero(imageBytes);
		// ********************* later version... **************	
		//PdfToPngConverter.closeDocument();
		System.exit(0);
	}

	@Override
	protected byte[] getPlainBytes() {
		return null;
	}

	/**
	 * Set an new list of image file names. Already existing list will be
	 * overwritten
	 * 
	 * @param newList	the new list
	 */
	public static void setFileList(ArrayList <String> newList) {
		fileList = newList;
	}
	/**
	 * Get the list of image file names
	 * 
	 * @return	the list of image file names
	 */
	public static ArrayList<String> getFileList() {
		return fileList;
	}
	/**
	 * Add a file name to the list of image file names
	 * 
	 * @param newFileName	the file name of the image file
	 */
	public static void addFileToList(String newFileName) {
		if (fileList == null) {
			fileList = new ArrayList<String>();
		}
		fileList.add(newFileName);
	}
	
/*	@Override
	public void adjustmentValueChanged(AdjustmentEvent ae) {        
		
		int extent = verticalBar.getModel().getExtent();
		int value = verticalBar.getValue();
		int max = verticalBar.getMaximum();*/
//		File pdfFile = new File("resources" + File.separator + "catena-v3.3.pdf");

       // System.out.println("extent: " + extent);
       // System.out.println("Value: " + (scrollPane.getVerticalScrollBar().getValue()+extent) + " Max: " + scrollPane.getVerticalScrollBar().getMaximum() + " , Min: " + scrollPane.getVerticalScrollBar().getMinimum());
       // messageLabel.setText("rendering next pages111111111...");
//        if (value + extent == max) {
        	
        	// load next png:
        	
/*        	//	File pdfFile = new File("resources" + File.separator + "Handbook of applied cryptography.pdf");
    		//	File pdfFile = new File("resources" + File.separator + "Balloon-functions.pdf");
        	setMessage("Xrendering next pages...");
        	messageLabel.setText("rendering next pages...");
        	System.out.println("up");
        	
    		page += pageNumber;
    		boolean success = PdfToPngConverter.convert(page, page + pageNumber);
    			if (success == true) {
    				
        			this.dispose();

        			LockFrameImage.setImageBytes(ReadResources.readExternFile(pdfFile.getPath().substring(0,  pdfFile.getPath().length() -3) + "png"));

        			LockFrameImage frame = new LockFrameImage(new Point(100,100));
        			frame.setVisible(true);
    			} else {
    				messageLabel.setText("end of document");
    				System.out.println("end of document");
    			}*/
//        } else if  (scrollPane.getVerticalScrollBar().getValue() == 0) {
        	
        	// load previous png:
        	

   /*     	messageLabel.setText("rendering previous pages...");
    		if (page == 0) {
    			messageLabel.setText("begin of document");
    			System.out.println("begin of document");
    		} else {
    			setMessage("rendering...");
    			page -= pageNumber;
    			if (pageNumber < 0) {    		
    				messageLabel.setText("go to begin of document...");
    				System.out.println("go to begin of document");
    				page = 0;
    			} else {
    				messageLabel.setText("rendering previous pages");
    				if (PdfToPngConverter.convert(page, page + pageNumber) == true) {
        				
    	       			this.dispose();

            			LockFrameImage.setImageBytes(ReadResources.readExternFile(pdfFile.getPath().substring(0,  pdfFile.getPath().length() -3) + "png"));
            			LockFrameImage frame = new LockFrameImage(new Point(100,100));
            			scrollPane.getVerticalScrollBar().setValue(5);
            			frame.setVisible(true);
        			} else {
        				System.out.println("end of document");
        			}
    			}
    		}*/
 //       }
//	} 
}