package cologne.eck.peafactory.tools;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.ArrayList;

import javax.swing.JOptionPane;

import cologne.eck.peafactory.peas.PswDialogBase;
import cologne.eck.peafactory.peas.gui.PswDialogView;






public class FileTools {
	
	private final static int NUMBER_LIMIT = 256; // Warning if selected file number > NUMBER_LIMIT
	private final static int EXTREME_NUMBER_LIMIT = 512; 
	private final static long SIZE_LIMIT = 1024 * 1024 * 64; // 64 MiB	
	private final static long EXTREME_SIZE_LIMIT = 1024 * 1024 * 1024; // 1 GiB
	
	/*
	 * true if a warning (long execution time because of size) was already shown
	 * (this warning should appear at most once for each session)
	 */
	private static boolean sizeWarning = false;
	/*
	 * true if a warning (extreme long execution time) was already shown
	 * (this warning should appear at most once for each session)
	 */
	private static boolean extremeSizeWarning = false;
	/*
	 * true if a warning (long execution time because of number of files) was already shown
	 * (this warning should appear at most once for each session)
	 */
	private static boolean numberWarning = false;
	/*
	 * true if a warning (extreme long execution time because of number of files) was already shown
	 * (this warning should appear at most once for each session)
	 */
	private static boolean extremeNumberWarning = false;
	
/*	private boolean read_write_failed = false;
	private boolean unexpected_error = false;
	private boolean file_identifier_failed = false;
	private boolean no_included_children = false;
	private boolean double_check = false;
	private boolean file_not_found = false;
	private boolean empty_file = false;
	private String unexpectedError = ""; */
	
	/**
	 * RandomAccessFile for basic file operations 
	 * (check file identifier, length...)
	 */
	private static RandomAccessFile raf;
	
	
	/**
	 * Get all files of a directory including sub-directories
	 * as an unsorted list. 
	 * 
	 * @param directory			the directory
	 * @param filesOnly			true if only file should be included
	 * 							false if files and directories should be included
	 * 
	 * @return	an unsorted list of all included files 
	 * 			or null if the directory is null 
	 * 			or is not to a directory 
	 */
	public static ArrayList <File> listAllFilesOfFolder(File directory, boolean filesOnly) {
		
		if (directory == null) {
			System.err.println("FileTools: directory is null");
			return null;
		}
		if (! directory.isDirectory()) {
			System.err.println("FileTools: argument is not directory");
			return null;		
		}
		ArrayList<File> resultList = new ArrayList<File>();

        File[] fileList = directory.listFiles();        
        int len = fileList.length;
        for (int i = 0; i < len; i++) {
        	File file = fileList[i];
        	//resultList.add(file);
            if (file.isFile()) {
            	resultList.add(file);
                //System.out.println(file.getAbsolutePath());
            } else if (file.isDirectory()) {
            	if (filesOnly == false){
            		resultList.add(file);
            	}
                resultList.addAll(listAllFilesOfFolder(file, filesOnly));
            }
        }
        return resultList;
	}
	
	/**
	 * Get all names of files of a directory including sub-directories
	 * as an unsorted list. 
	 * 
	 * @param directoryName		the name of the directory
	 * @param filesOnly			true if only file should be included
	 * 							false if files and directories should be included
	 * 
	 * @return	an unsorted list of all included file names 
	 * 			or null if the directory name is null 
	 * 			or refers not to a directory 
	 */
	public static ArrayList <String> listAllFileNamesOfFolder(String directoryName, boolean filesOnly) {
		
		if (directoryName == null) {
			System.err.println("FileTools: directory nam is null");
			return null;
		}
		File directory = new File(directoryName);
		if (! directory.isDirectory()) {
			System.err.println("FileTools: argument is not directory");
			return null;		
		}
		ArrayList<String> resultList = new ArrayList<String>();

        File[] fileList = directory.listFiles();        
        int len = fileList.length;
        for (int i = 0; i < len; i++) {
        	File file = fileList[i];
        	resultList.add(file.getAbsolutePath());
            if (file.isFile()) {
            	resultList.add(file.getAbsolutePath());
                //System.out.println(file.getAbsolutePath());
            } else if (file.isDirectory()) {
            	if (filesOnly == false){
            		resultList.add(file.getAbsolutePath());
            	}
                resultList.addAll(listAllFileNamesOfFolder(file.getAbsolutePath(), filesOnly));
            }
        }
        return resultList;
	}
	/**
	 * Get all names of files of a directory including sub-directories
	 * as an array of Strings. 
	 * 
	 * @param directoryName		the name of the directory
	 * @param filesOnly			true if only file should be included
	 * 							false if files and directories should be included
	 * 
	 * @return	an array of all included file names 
	 * 			or null if the directory name is null 
	 * 			or refers not to a directory 
	 */
	public static String[] getAllFileNamesOfFolder(String directoryName, boolean filesOnly) {
		ArrayList <String> list = listAllFileNamesOfFolder(directoryName, filesOnly);
		if (list == null) {
			return null;
		} else {
			String[] array = new String[list.size()];
			return list.toArray(array);
		}
	}
	/**
	 * Get all files of a directory including sub-directories
	 * as an array of Strings. 
	 * 
	 * @param directoryName		the directory
	 * @param filesOnly			true if only file should be included
	 * 							false if files and directories should be included
	 * 
	 * @return	an array of all included files 
	 * 			or null if the directory name is null 
	 * 			or refers not to a directory 
	 */
	public static File[] getAllFilesOfFolder(File directory, boolean filesOnly) {
		ArrayList <File> list = listAllFilesOfFolder(directory, filesOnly);
		if (list == null) {
			return null;
		} else {
			File[] array = new File[list.size()];
			return list.toArray(array);
		}
	}
	
	/**
	 *  Get the number of files of a directory - inclusive 
	 *  all files of sub-directories, but no directory itself
	 * 
	 * @param directoryName		the name of the directory to check
	 * @return					an array of long values: the first value [0] is the
	 * 							number of the containing files (no directories), 
	 * 							the second [1] is the size of all included files
	 * 							or null if user wanted to break the operation
	 */
	public static final long[] getNumberAndSize(String directoryName) {
		
		File directory = new File(directoryName);

		long[] result = new long[2];

        File[] children = directory.listFiles();        
        if (children != null) {        	

            for (int i = 0; i < children.length; i++) {
            	if (children[i].isDirectory() ) {
            		long[] dirResult = getNumberAndSize(children[i].getAbsolutePath());
            		// function was broken because of extremeSizeWarning
            		// this avoids NullPointerException
            		if (dirResult == null) {
            			return null;
            		}
            		result[0] += dirResult[0];
            		result[1] += dirResult[1];    				
 
            	} else if (children[i].isFile() ) {

            		try {            			
						raf = new RandomAccessFile(children[i], "r");	            		
	            		result[1] += raf.length();
	            		raf.close(); 
					} catch (FileNotFoundException e) {
						//file_not_found = true;
						System.err.println("FileTools getNumberAndSize: " +  e.getMessage() + ", " + children[i]);
						//e.printStackTrace();
					} catch (IOException e) {
						//read_write_failed = true;
						System.err.println("FileTools getNumberAndSize: " +  e.getMessage() + ", " + children[i]);
						//e.printStackTrace();
					} catch (Exception e) {
						System.err.println(e.toString() + " - " + e.getMessage() + ", " + children[i]);
						//e.printStackTrace();								
					}
            		// only count files, not folders:
            		result[0] ++;
            	}
           		// check number of files and size 
        		
       			String warningMessage = null;
           		
    			if (result[0] > EXTREME_NUMBER_LIMIT && extremeNumberWarning == false) {
    				warningMessage = PswDialogBase.getBundle().getString("extrem_number_warning");
    				extremeNumberWarning = true;
    			} else if (result[0] > NUMBER_LIMIT &&  numberWarning == false) {
    				warningMessage = PswDialogBase.getBundle().getString("number_warning");
    				numberWarning = true;
    			}
    			if (result[1] > EXTREME_SIZE_LIMIT && extremeSizeWarning == false) {
    				warningMessage = PswDialogBase.getBundle().getString("extrem_size_warning");
    				extremeSizeWarning = true;
    			} else if (result[1] > SIZE_LIMIT && sizeWarning == false){
    				warningMessage = PswDialogBase.getBundle().getString("size_warning");
    				sizeWarning = true;
    			}
 
   				if (warningMessage != null) {
					
	   		   		Object[] options = {PswDialogBase.getBundle().getString("go_on"),
    	    				PswDialogBase.getBundle().getString("cancel")};
					
	   		   		int n = JOptionPane.showOptionDialog(PswDialogView.getView(),
    					warningMessage,
    						PswDialogBase.getBundle().getString("warning"),
    						JOptionPane.YES_NO_OPTION,
    						JOptionPane.QUESTION_MESSAGE,
    						null,
    						options,
    						options[0]);

	   		   		if (n == 0) {
    			 		//
	   		   		} else { // cancel or close
	   		   			return null;
	   		   		}
				}
            }
        }       
        return result;
    }
}
