package cologne.eck.peafactory.tools;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.rendering.PDFRenderer;

//import cologne.eck.peafactory.peas.gui.PswDialogView;



public class PdfToPngConverter {
	

	private float scale = 2; // quality
	
	private File pdfFile = null;
	private PDDocument doc = null;
	private int allPages = 0;
	


	public PdfToPngConverter(File _pdfFile) {
		this.pdfFile = _pdfFile;
		try {
			doc = PDDocument.load(pdfFile);
			allPages = doc.getNumberOfPages();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
/*	public void setDocument(File _pdfFile) {

		try {
			pdfFile = _pdfFile;
			doc = PDDocument.load(pdfFile);
			allPages = doc.getNumberOfPages();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}*/
	
	public void closeDocument() {
		
		try {
			this.doc.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
	}
	
	public void convert() {
/*		PDDocument doc = null;
		try {
			doc = PDDocument.load(pdfFile);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}*/
		int lastPage = allPages;
		convert(0, lastPage);
	}
	
	public boolean convert(int firstPage, int lastPage) {
		
		//( (LockFrameImage) PswDialogBase.getDialog().getLockFrame()).setMessage("Xrendering next pages...");
/*		try{
			( (LockFrameImage) PswDialogBase.getDialog().getLockFrame()).setMessage("Xrendering next pages...");
		} catch (NullPointerException e){
			System.out.println("null");
		} */
		

		//int pageNumber = doc.getNumberOfPages();
		int pageNumber = lastPage - firstPage;
		//PDPage page = doc.getPage(0);
		
		PDFRenderer renderer = new PDFRenderer(doc);
		
		int width = 400;

		int height = 0;

		int margin = 50;
		

/*		// this is faster but less accurate: 
 		// accurate only if all page are the same size
  		// and 
		// throws a java.lang.OutOfMemoryError: Java heap space
		// for the same number of pages
		int pageHeight = 0;
		int pageWidth = 0;
		BufferedImage tmpPage = null;//[pageNumber];
		try {
			tmpPage = renderer.renderImage(0, scale);
			pageWidth = tmpPage.getWidth();
			pageHeight = tmpPage.getHeight();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		width = pageWidth;
		height += (pageHeight * pageNumber) + (margin * pageNumber) + margin;
		BufferedImage image = null;
		try{
			image =  new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
		} catch (java.lang.OutOfMemoryError x){
			JOptionPane.showMessageDialog(PswDialogView.getView(), "Too many pages.");
		}
		Graphics2D g2d = image.createGraphics();
		int y = margin;
		for (int i = firstPage; i < lastPage; i++) {		
			try {
				g2d.drawImage(renderer.renderImage(i, scale), 0, y, null);		
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			y += pageHeight + margin;
		}
		g2d.dispose();  */


		BufferedImage[] pages = new BufferedImage[pageNumber];
		// register if end of document is reached:
		if ( firstPage + pageNumber > allPages ) {
			pageNumber = allPages - firstPage;
		}
		try {
			// get the required width and height			
			for (int i = 0; i < pageNumber; i++) {
				pages[i] = renderer.renderImage(i + firstPage, scale); // page, scale
				if (pages[i].getWidth() > width) {
					width = pages[i].getWidth();
				}
				height += pages[i].getHeight();
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		
		// margin between pages:
		height += margin * pageNumber + margin;

		BufferedImage image = null;
		if (height < 0) { // end of document
			return false;
		}
		image =  new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);

		Graphics2D g2d = image.createGraphics();
		

		int y = margin;
		// draw the image: append the pages
		for (int i = 0; i < pageNumber; i++) {			
			g2d.drawImage(pages[i], 0, y, null);
			// get the next y
			y += pages[i].getHeight() + margin;
		}

		g2d.dispose();
		
		try {
			ImageIO.write(image, "PNG", new File(pdfFile.getPath().substring(0,  pdfFile.getPath().length() -3) + "png"));
	//		ImageIO.write(image, "PNG", new File("/home/axl/all/prog/eclipse-work/cologne.eck.peafactory_0.1/resources/Bewerberbroschuere.png"));
			System.out.println(pdfFile.getName());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
		return true;
	}

	/**
	 * @return the scale
	 */
	public float getScale() {
		return scale;
	}

	/**
	 * @param scalex the scale to set
	 */
	public void setScale(float newScalex) {
		this.scale = newScalex;
	}
}
